#!/usr/bin/perl -w

package RPM::Source::Convert::ALTLinuxBackport;
use strict;
use warnings;
use Carp;
use Getopt::Long;
use RPM;
use RPM::Source::Transform 0.010;

use RPM::Source::Convert;
use RPM::Source::Transformation::RaiseReleaseFactory;
use RPM::Source::Convert::Plugins::DistroMap::Groups;
use RPM::Source::Convert::Plugins::DistroMap::Dependencies;

$RPM::Source::Transformation::RaiseReleaseFactory::nextrel='altbackport';
$RPM::Source::Transformation::RaiseReleaseFactory::defaultchangelog='- backport';
$RPM::Source::Transformation::RaiseReleaseFactory::force_transform_mode=1;
&RPM::Source::Transformation::RaiseReleaseFactory::register_nextrel_method(
    -name => 'altbackport',
    -nextrel_method => \&nextrel_altbackport,
    -initrel_method => \&nextrel_altbackport_init,
    );

$DistroMap::default_originrepo='altlinux';
$DistroMap::default_originbranch='sisyphus';
$DistroMap::default_destrepo='altlinux';
$DistroMap::default_destbranch='p6';

our $backport_suffix='M60P';

push @RPM::Source::Transform::EXT_OPTS_USAGE, \&longopt_usage;
push @RPM::Source::Transform::EXT_OPTS,
    "A=s"   => \&__set_options,
    "M=s"   => \&__set_options;

sub __set_options {
    my ($bpclass,$suf)=@_;
    $backport_suffix=join('',$bpclass,$suf);
    my $destbranch=&_suffix2destbranch($backport_suffix);
    $DistroMap::default_destbranch=$destbranch;
    print STDERR "backport: destbranch=$destbranch suffix=$backport_suffix\n" if $RPM::Source::Transform::verbose>1;
}

sub longopt_usage {
    my ($self)=@_;
    print "ALT Linux backport specific options:\n";
    if ($Getopt::Long::bundling) {
	print "	-Mxx -- branch to backport. For example, -M60P.\n";
	print "	-Axx -- branch to autoport. For example, -A60T.\n";
    } else {
	print "	-M xx -- branch to backport. For example, -M 60P.\n";
	print "	-A xx -- branch to autoport. For example, -A 60T.\n";
    }
}

sub __get_backport_suffix() {
    my ($self)=@_;
    return $backport_suffix;
}

sub nextrel_altbackport {
    my ($rel,$spec,$parent)=@_;
    # $rel!~/\%{?release}?/; otherwise it is recurrent release invocation
    $spec->macros()->macro_subst_inplace(\$rel) if $rel=~/\%/ and $rel!~/\%{?release}?/;
    # $rel=~s/\%branch_release\s*//;
    # need after macro_subst_inplace
    $rel=~s/^\s*//;
    return &_nextrel_altbackport_raw($rel, $backport_suffix);
}

sub nextrel_altbackport_init {
    my ($spec,$parent)=@_;
    my $rel=$spec->main_section->get_tag('Release');
    # need after macro_subst_inplace
    $rel=~s/^\s*//;
    return &_nextrel_altbackport_raw($rel, $backport_suffix);
}

sub _nextrel_altbackport_raw {
    my ($rel, $backportsuffix)=@_;
    die "bad Release: [$rel] (does not start with alt - not an ALT Linux release !)" unless $rel=~/^(?:alt|\%)/;
    die "bad Release: [$rel] (does not have a digit - not an ALT Linux release !)" unless $rel=~/^(?:alt.*\d|\%).*$/;
    my $newrel = 'alt0.'.$backportsuffix.'.1';
    my ($relmajor,$relminor);
    if ($rel=~/^alt(.+\.|)(\d+)$/) {
	$relmajor=$1;
	$relminor=$2;
    } elsif($rel=~/^alt(.+)(\d+)$/) {
	$relmajor=$1;
	$relminor=$2;
    }
    if ($relminor) {
	$newrel = 'alt'.$relmajor.($relminor-1).'.'.$backportsuffix.'.1';
    }
    return $newrel;
}

sub _suffix2destbranch {
    my ($suffix)=@_;
    my $destbranch='p6';
    unless ($suffix=~/^([ABM])(.+)$/) {
	print STDERR "WARNING: unsupported suffix $suffix\n";
	return $destbranch;
    }
    my ($bpclass,$suf)=($1,$2);
    if ($suf eq '60' and $bpclass eq 'A') {
	$destbranch='t6';
    } elsif ($suf =~ '^([56789])0([PT])$') {
	$destbranch="\L$2\E${1}";
    } elsif ($suf =~ '^([5432])([012])$') {
	$destbranch="$1.$2";
    } else {
	print STDERR "WARNING: unsupported suffix $suffix\n";
    }
    return $destbranch;
}

sub _destbranch2suffix {
    my ($branch)=@_;
    my $suffix='M60P';
    if ($branch =~ '^([pt])([56789])$') {
	$suffix="M${2}0\U$1\E";
    } elsif ($branch =~ '^([5432])\.([012])$') {
	$suffix="M$1$2";
    } else {
	print STDERR "WARNING: unsupported branch $branch\n";
    }
    return $suffix;
}

sub __backport_cmp_origin_dest {
    my ($origin,$dest,$backport_suffix,$verbose) =@_;
    return 1 unless $dest;
    return -1 unless $origin;
    my $originversion=$origin->version;
    my $originrelease=$origin->release;
    my $destversion=$dest->version;
    my $destrelease=$dest->release;
    my $autoportsrelease=&RPM::Source::Convert::ALTLinuxBackport::_nextrel_altbackport_raw($originrelease, $backport_suffix);
    my $cmp = $originversion cmp $destversion;
    if ($cmp!=0) {
	$cmp=RPM::evrcmp($originversion,$destversion);
	print STDERR "version: $originversion rpmcmp $destversion = $cmp\n" if $verbose>1;
    }
    return $cmp if $cmp !=0;
    return 0 if $destrelease eq $autoportsrelease or $destrelease eq $originrelease;
    # Binary "cmp" returns -1, 0, or 1 depending on whether the left argument
    # is stringwise less than, equal to, or greater than the right argument.
    $cmp=RPM::evrcmp($originrelease,$destrelease);
    print STDERR "$originrelease rpmcmp $destrelease = $cmp\n" if $verbose>1;
    return $cmp if $cmp <= 0;
    $cmp=RPM::evrcmp($autoportsrelease,$destrelease);
    print STDERR "$autoportsrelease rpmcmp $destrelease = $cmp\n" if $verbose>1;
    return $cmp;
}

1;
