#!/usr/bin/perl -w

package RPM::Source::Convert::Plugins::DistroMap::Groups;
use strict;
use warnings;
use Carp;

use RPM::Source::Transform;
use RPM::Source::Transformation::Component;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Transformation::DistroMapResource;
our @ISA=qw/RPM::Source::Transformation::Component/;

push @RPM::Source::Transform::FACTORY_CLASSES, __PACKAGE__;

our %ALLOWED_GROUP;
# generic conversions
our %EXTRA_GROUP_POSTMAP = ();
# specific conversions
our %EXTRA_GROUP_PREMAP = ();

our $default_group='Other';
# TODO
my $opt_default_group;

sub longopt_usage {
    print "DistroMap::Groups options:\n";
    print "	--default-group <default group>. Default is $default_group.\n";
}

our @LONGOPT=(
    'default-group=s' => \$default_group,
    );

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'distromap_group_convert',
	    -opt_handler_sub=>\&opt_handler_distromap_group_convert),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_PRERELEASE,
	    -name => 'distromap_group_posthook',
	    -opt_handler_sub=>\&opt_handler_distromap_group_posthook);
}

sub init_factory {
    my ($class,$resourceobj)=@_;
    return if %ALLOWED_GROUP;

    # chroot build; no distromap
    unless ($ENV{'RPM_BUILD_ROOT'}) {
	my $distromap=$resourceobj->get_distromap;
	if ($distromap) {
	    # TODO
	    my $dest=$distromap->dest;
	    # TODO: need an interface to load a list
	    # 'groups-allowed'
	}
	return if %ALLOWED_GROUP;
    }

    if (-e '/usr/lib/rpm/GROUPS') {
	open my $rpmgroup, '<' , '/usr/lib/rpm/GROUPS' || die "can't open /usr/lib/rpm/GROUPS: $!";
	my $group;
	while ($group=<$rpmgroup>) {
	    chomp $group;
	    $group=~s/\s*$//;
	    $ALLOWED_GROUP{$group}=1 if $group;
	}
	close $rpmgroup;
    }
    return RPM::Source::Transformation::Component::TRANSFORMATION_OK;
}

# todo: process/verify_options callback sub? or init will do?
#$opt_default_group||=$default_group

sub opt_handler_distromap_group_posthook {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    my $mainsec=$spec->main_section;

    # take a missing group from parent
    if ($parent) {
	foreach my $sec ($spec->get_sections()) {
	    if ($sec->get_type eq 'package' and not $sec->get_tag('Group')) {
		my $p = $sec->get_raw_package();
		my $parentsec=$parent->get_section('package',$p);
		next unless $parentsec;
		my $parentgroup=$parentsec->get_tag('Group');
		next unless $parentgroup;
		$sec->set_tag('Group',$parentgroup);
	    }
	}
    }

    unless (defined $mainsec->get_tag('Group')) {
	warn "WARNING: main section does not contain a Group: tag. Using $default_group\n";
	$mainsec->set_tag('Group',$default_group);
    }

    my $distromap=$resourceobj->get_distromap;
    foreach my $sec ($spec->get_sections()) {
	&distromap_group_postmap($distromap,$sec,$spec,$parent) if $sec->get_type eq 'package';
    }

    my $maingroup=$spec->main_section->get_tag('Group');
    # generic_merge_main_group_if_group_is_missing to avoid
    # error: Group field must be present in package: <subpackage>
    foreach my $sec ($spec->get_sections()) {
	$sec->set_tag('Group',$maingroup) if $sec->get_type eq 'package' and not $sec->get_tag('Group');
    }
    return RPM::Source::Transformation::Component::TRANSFORMATION_OK;
}

sub opt_handler_distromap_group_convert {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    my $distromap=$resourceobj->get_distromap;
    foreach my $sec ($spec->get_sections()) {
	&distromap_group_premap($distromap,$spec,$sec) if $sec->get_type eq 'package';
    }
    return RPM::Source::Transformation::Component::TRANSFORMATION_OK;
}

sub distromap_group_premap {
    my ($distromap,$spec,$section)=@_;
    my @out;
    foreach (@{$section->get_bodyref()}) {
	if (/^Group:(\s*)(\S.*\S)\s*$/) {
	    my $gspace=$1;
	    my $group=$2;
	    my $group_premap=$distromap->unimap_exact_group_origin2dest($group);
	    $group_premap=$EXTRA_GROUP_PREMAP{$group} unless $group_premap;
	    if ($group_premap) {
		push @out, "Group:$gspace$group_premap\n";
	    } else {
		push @out, $_;
	    }
	} else {
	    push @out, $_;
	}
    }
    $section->set_body(\@out);
}

sub distromap_group_postmap {
    my ($distromap,$section,$spec,$parent)=@_;
    my @out;
    foreach (@{$section->get_bodyref()}) {
	if (/^Group:(\s*)(\S.*\S)\s*$/) {
	    my $gspace=$1;
	    my $group=$2;
	    my $newgroup;
	    unless ($ALLOWED_GROUP{$group}) {
		if ($parent) {
		    my $parentsec=$parent->get_section('package',$section->get_raw_package());
		    $newgroup||=$parentsec->get_tag('Group') if $parentsec;
		}
		unless ($newgroup) {
		    my $sec_pkg_expanded = $section->get_package_name();
		    if ($sec_pkg_expanded =~/^python\d?-module/) {
			&__try_group(\$newgroup,'Development/Python');
		    } elsif ($sec_pkg_expanded =~/^perl-/) {
			&__try_group(\$newgroup,'Development/Perl');
		    } elsif ($sec_pkg_expanded =~/^fonts-[ot]tf-/) {
			&__try_group(\$newgroup,'System/Fonts/True type');
		    } elsif ($sec_pkg_expanded =~/^fonts-type1-/) {
			&__try_group(\$newgroup,'System/Fonts/Type1');
		    } elsif ($sec_pkg_expanded =~/^fonts-bitmap-/) {
			&__try_group(\$newgroup,'System/Fonts/X11 bitmap');
		    } elsif ($sec_pkg_expanded =~/^firmware-/ or $sec_pkg_expanded =~/-firmware$/) {
			&__try_group(\$newgroup,'System/Kernel and hardware');
		    }
		}
		unless ($newgroup) {
		    my $group_postmap=$distromap->unimap_approximate_group_origin2dest($group);
		    $group_postmap=$EXTRA_GROUP_POSTMAP{$group} unless $group_postmap;
		    print STDERR "warning: substituting Group: $group with unaccurate $group_postmap\n" if $group_postmap;
		    $newgroup||=$group_postmap;
		}
		unless ($newgroup) {
		    my $pkg=$section->get_raw_package();
		    $pkg||='main section';
		    print STDERR "WARNING: forbidden Group: $group in $pkg. Please, replace.\n";
		}
	    }

	    if ($newgroup) {
		push @out, "Group:$gspace$newgroup\n";
	    } else {
		push @out, $_;
	    }
	} else {
	    push @out, $_;
	}
    }    
    $section->set_body(\@out);
}

sub __try_group {
    my ($ptr, $name)=@_;
    $$ptr=$name if $ALLOWED_GROUP{$name};
}

1;
